@@ -111,11 +111,9 @@ cell_properties_set_table (GtkhtmlEditor *editor,
 {
 	GtkHTML *html;
 	HTMLObject *parent;
-	HTMLTableCell *cell;
 	HTMLTableCell *iter;
 
 	html = gtkhtml_editor_get_html (editor);
-	cell = HTML_TABLE_CELL (editor->priv->cell_object);
 	parent = editor->priv->cell_parent;
 
 	iter = html_engine_get_table_cell (html->engine);
@@ -253,7 +251,7 @@ cell_properties_set_horizontal_alignment_cb (GtkhtmlEditor *editor,
 	active = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
 	align = HTML_HALIGN_LEFT + (HTMLHAlignType) active;
 
-	html_engine_table_cell_set_halign (html->engine, cell, active);
+	html_engine_table_cell_set_halign (html->engine, cell, align);
 }
 
 static void
@@ -1014,6 +1012,23 @@ gtkhtml_editor_image_properties_url_entry_changed_cb (GtkWidget *window))
  * Link Properties Window
  *****************************************************************************/
 
+static gchar *
+sanitize_description_text (const gchar *ptext)
+{
+	gchar *p, *text = g_strdup (ptext);
+
+	if (!text)
+		return text;
+
+	/* replace line breakers with spaces */
+	for (p = text; *p; p++) {
+		if (*p == '\r' || *p == '\n')
+			*p = ' ';
+	}
+
+	return g_strstrip (text);
+}
+
 AUTOCONNECTED_SIGNAL_HANDLER (void
 gtkhtml_editor_link_properties_description_changed_cb (GtkWidget *window))
 {
@@ -1029,37 +1044,31 @@ gtkhtml_editor_link_properties_description_changed_cb (GtkWidget *window))
 	dsc_entry = WIDGET (LINK_PROPERTIES_DESCRIPTION_ENTRY);
 	url_entry = WIDGET (LINK_PROPERTIES_URL_ENTRY);
 
-	text = g_strstrip (g_strdup (
-		gtk_entry_get_text (GTK_ENTRY (dsc_entry))));
+	text = sanitize_description_text (gtk_entry_get_text (GTK_ENTRY (dsc_entry)));
 
 	editor->priv->link_custom_description = (*text != '\0');
 
 	if (editor->priv->link_custom_description) {
-		gint start_offset;
-		gint end_offset;
 		glong length;
+		Link *link;
 
-		start_offset = editor->priv->link_start_offset;
-		end_offset = editor->priv->link_end_offset;
+		link = html_text_get_link_at_offset (HTML_TEXT (html->engine->cursor->object), html->engine->cursor->offset);
 		length = g_utf8_strlen (text, -1);
 
-		if (start_offset != end_offset) {
+		if (link && link->start_offset != link->end_offset) {
 			html_cursor_jump_to (
 				html->engine->cursor, html->engine,
-				editor->priv->link_object, start_offset);
+				html->engine->cursor->object, link->start_offset);
 			html_engine_set_mark (html->engine);
 			html_cursor_jump_to (
 				html->engine->cursor, html->engine,
-				editor->priv->link_object, end_offset);
+				html->engine->cursor->object, link->end_offset);
 			html_engine_delete (html->engine);
 		}
 
 		html_engine_paste_link (
 			html->engine, text, length,
 			gtk_entry_get_text (GTK_ENTRY (url_entry)));
-
-		editor->priv->link_object = html->engine->cursor->object;
-		editor->priv->link_end_offset = start_offset + length;
 	}
 
 	g_free (text);
@@ -1090,34 +1099,31 @@ gtkhtml_editor_link_properties_url_changed_cb (GtkWidget *window))
 	if (html_engine_is_selection_active (html->engine)) {
 		html_engine_set_link (html->engine, text);
 	} else if (!editor->priv->link_custom_description) {
-		gtk_entry_set_text (GTK_ENTRY (dsc_entry), text);
+		gchar *descr = sanitize_description_text (text);
+		gtk_entry_set_text (GTK_ENTRY (dsc_entry), descr);
+		g_free (descr);
 		editor->priv->link_custom_description = FALSE;
 	} else {
-		gint start_offset;
-		gint end_offset;
 		glong length;
+		Link *link;
 		const gchar *descr = gtk_entry_get_text (GTK_ENTRY (dsc_entry));
 
-		start_offset = editor->priv->link_start_offset;
-		end_offset = editor->priv->link_end_offset;
+		link = html_text_get_link_at_offset (HTML_TEXT (html->engine->cursor->object), html->engine->cursor->offset);
 		length = g_utf8_strlen (descr, -1);
 
-		if (start_offset != end_offset) {
+		if (link && link->start_offset != link->end_offset) {
 			html_cursor_jump_to (
 				html->engine->cursor, html->engine,
-				editor->priv->link_object, start_offset);
+				html->engine->cursor->object, link->start_offset);
 			html_engine_set_mark (html->engine);
 			html_cursor_jump_to (
 				html->engine->cursor, html->engine,
-				editor->priv->link_object, end_offset);
+				html->engine->cursor->object, link->end_offset);
 			html_engine_delete (html->engine);
 		}
 
 		html_engine_paste_link (
 			html->engine, descr, length, text);
-
-		editor->priv->link_object = html->engine->cursor->object;
-		editor->priv->link_end_offset = start_offset + length;
 	}
 
 	g_free (text);
@@ -1135,8 +1141,6 @@ gtkhtml_editor_link_properties_show_window_cb (GtkWidget *window))
 	gchar *url = NULL, *dsc = NULL;
 	gboolean sensitive;
 	HTMLCursor *cursor;
-	gint start_offset = 0;
-	gint end_offset = 0;
 
 	editor = extract_gtkhtml_editor (window);
 	html = gtkhtml_editor_get_html (editor);
@@ -1153,25 +1157,17 @@ gtkhtml_editor_link_properties_show_window_cb (GtkWidget *window))
 			cursor->object, cursor->offset);
 
 	if (url != NULL) {
-		if (HTML_IS_IMAGE (cursor->object)) {
-			start_offset = 0;
-			end_offset = 1;
-		} else {
+		if (!HTML_IS_IMAGE (cursor->object)) {
 			Link *link;
 
 			link = html_text_get_link_at_offset (
 				HTML_TEXT (cursor->object),
 				cursor->offset);
 			if (link != NULL) {
-				start_offset = link->start_offset;
-				end_offset = link->end_offset;
 				dsc = html_text_get_link_text (HTML_TEXT (cursor->object), cursor->offset);
 				editor->priv->link_custom_description = dsc && !g_str_equal (dsc, url);
 			}
 		}
-	} else if (HTML_IS_TEXT (cursor->object)) {
-		start_offset = cursor->offset;
-		end_offset = start_offset;
 	}
 
 	sensitive = (url == NULL);
@@ -1182,15 +1178,18 @@ gtkhtml_editor_link_properties_show_window_cb (GtkWidget *window))
 		sensitive = FALSE;
 	}
 
-	editor->priv->link_object = cursor->object;
-	editor->priv->link_start_offset = start_offset;
-	editor->priv->link_end_offset = end_offset;
-
 	gtk_widget_set_sensitive (dsc_entry, sensitive);
 	gtk_entry_set_text (
 		GTK_ENTRY (url_entry),
 		(url != NULL) ? url : "http://");
 
+	if (dsc) {
+		gchar *tmp = dsc;
+
+		dsc = sanitize_description_text (dsc);
+		g_free (tmp);
+	}
+
 	gtk_entry_set_text (GTK_ENTRY (dsc_entry), dsc ? dsc : "");
 	gtk_widget_grab_focus (url_entry);
 
